@@ -96,10 +96,9 @@
     #include <utmpx.h>
     #include <sys/vnode.h>  // for VREG
     #include <sys/sched.h>  // for CPUSTATES & CP_*
-    #include <machine/vmparam.h>  // for PAGE_SHIFT
-#define _KERNEL
-    #include <uvm/uvm_extern.h>
-#undef _KERNEL
+    #ifndef DTYPE_VNODE
+    #define DTYPE_VNODE 1
+    #endif
 #endif
 
 
@@ -192,14 +191,23 @@ static PyObject *
 psutil_proc_name(PyObject *self, PyObject *args) {
     long pid;
     kinfo_proc kp;
+    char str[1000];
+
     if (! PyArg_ParseTuple(args, "l", &pid))
         return NULL;
     if (psutil_kinfo_proc(pid, &kp) == -1)
         return NULL;
+
 #ifdef __FreeBSD__
-    return Py_BuildValue("s", kp.ki_comm);
+    sprintf(str, "%s", kp.ki_comm);
 #elif defined(__OpenBSD__) || defined(__NetBSD__)
-    return Py_BuildValue("s", kp.p_comm);
+    sprintf(str, "%s", kp.p_comm);
+#endif
+
+#if PY_MAJOR_VERSION >= 3
+    return PyUnicode_DecodeFSDefault(str);
+#else
+    return Py_BuildValue("s", str);
 #endif
 }
 
@@ -360,7 +368,7 @@ psutil_proc_num_ctx_switches(PyObject *self, PyObject *args) {
 static PyObject *
 psutil_proc_cpu_times(PyObject *self, PyObject *args) {
     long pid;
-    double user_t, sys_t;
+    double user_t, sys_t, children_user_t, children_sys_t;
     kinfo_proc kp;
     if (! PyArg_ParseTuple(args, "l", &pid))
         return NULL;
@@ -370,11 +378,18 @@ psutil_proc_cpu_times(PyObject *self, PyObject *args) {
 #ifdef __FreeBSD__
     user_t = PSUTIL_TV2DOUBLE(kp.ki_rusage.ru_utime);
     sys_t = PSUTIL_TV2DOUBLE(kp.ki_rusage.ru_stime);
+    children_user_t = PSUTIL_TV2DOUBLE(kp.ki_rusage_ch.ru_utime);
+    children_sys_t = PSUTIL_TV2DOUBLE(kp.ki_rusage_ch.ru_stime);
 #elif defined(__OpenBSD__) || defined(__NetBSD__)
     user_t = PSUTIL_KPT2DOUBLE(kp.p_uutime);
     sys_t = PSUTIL_KPT2DOUBLE(kp.p_ustime);
+    // OpenBSD and NetBSD provide children user + system times summed
+    // together (no distinction).
+    children_user_t = kp.p_uctime_sec + kp.p_uctime_usec / 1000000.0;
+    children_sys_t = children_user_t;
 #endif
-    return Py_BuildValue("(dd)", user_t, sys_t);
+    return Py_BuildValue("(dddd)",
+                         user_t, sys_t, children_user_t, children_sys_t);
 }
 
 
@@ -445,15 +460,12 @@ psutil_proc_io_counters(PyObject *self, PyObject *args) {
 }
 
 
-#if defined(__OpenBSD__) || defined(__NetBSD__)
-    #define ptoa(x) ((paddr_t)(x) << PAGE_SHIFT)
-#endif
-
 /*
  * Return extended memory info for a process as a Python tuple.
  */
 static PyObject *
 psutil_proc_memory_info(PyObject *self, PyObject *args) {
+    long pagesize = sysconf(_SC_PAGESIZE);
     long pid;
     kinfo_proc kp;
     if (! PyArg_ParseTuple(args, "l", &pid))
@@ -464,26 +476,27 @@ psutil_proc_memory_info(PyObject *self, PyObject *args) {
     return Py_BuildValue(
         "(lllll)",
 #ifdef __FreeBSD__
-        (long) ptoa(kp.ki_rssize),  // rss
+        (long) kp.ki_rssize * pagesize,  // rss
         (long) kp.ki_size,  // vms
-        (long) ptoa(kp.ki_tsize),  // text
-        (long) ptoa(kp.ki_dsize),  // data
-        (long) ptoa(kp.ki_ssize)  // stack
+        (long) kp.ki_tsize * pagesize,  // text
+        (long) kp.ki_dsize * pagesize,  // data
+        (long) kp.ki_ssize * pagesize  // stack
 #else
-        (long) ptoa(kp.p_vm_rssize),    // rss
+        (long) kp.p_vm_rssize * pagesize,    // rss
     #ifdef __OpenBSD__
         // VMS, this is how ps determines it on OpenBSD:
         // http://anoncvs.spacehopper.org/openbsd-src/tree/bin/ps/print.c#n461
-        (long) ptoa(kp.p_vm_dsize + kp.p_vm_ssize + kp.p_vm_tsize),  // vms
+        // vms
+        (long) (kp.p_vm_dsize + kp.p_vm_ssize + kp.p_vm_tsize) * pagesize,
     #elif __NetBSD__
         // VMS, this is how top determines it on NetBSD:
         // ftp://ftp.iij.ad.jp/pub/NetBSD/NetBSD-release-6/src/external/bsd/
         //     top/dist/machine/m_netbsd.c
-        (long) ptoa(kp.p_vm_msize),  // vms
+        (long) kp.p_vm_msize * pagesize,  // vms
     #endif
-        (long) ptoa(kp.p_vm_tsize),  // text
-        (long) ptoa(kp.p_vm_dsize),  // data
-        (long) ptoa(kp.p_vm_ssize)  // stack
+        (long) kp.p_vm_tsize * pagesize,  // text
+        (long) kp.p_vm_dsize * pagesize,  // data
+        (long) kp.p_vm_ssize * pagesize  // stack
 #endif
     );
 }
@@ -703,8 +716,10 @@ psutil_disk_partitions(PyObject *self, PyObject *args) {
             strlcat(opts, ",relatime", sizeof(opts));
         if (flags & MNT_IGNORE)
             strlcat(opts, ",ignore", sizeof(opts));
+#if defined(MNT_DISCARD)
         if (flags & MNT_DISCARD)
             strlcat(opts, ",discard", sizeof(opts));
+#endif
         if (flags & MNT_EXTATTR)
             strlcat(opts, ",extattr", sizeof(opts));
         if (flags & MNT_LOG)
@@ -804,7 +819,12 @@ psutil_net_io_counters(PyObject *self, PyObject *args) {
                                         if2m->ifm_data.ifi_ierrors,
                                         if2m->ifm_data.ifi_oerrors,
                                         if2m->ifm_data.ifi_iqdrops,
-                                        0);  // dropout not supported
+#ifdef _IFI_OQDROPS
+                                        if2m->ifm_data.ifi_oqdrops
+#else
+                                        0
+#endif
+                                        );
             if (!py_ifc_info)
                 goto error;
             if (PyDict_SetItemString(py_retdict, ifc_name, py_ifc_info))
@@ -998,6 +1018,8 @@ PsutilMethods[] = {
      "Return a Python dict of tuples for disk I/O information"},
     {"users", psutil_users, METH_VARARGS,
      "Return currently connected users as a list of tuples"},
+    {"cpu_stats", psutil_cpu_stats, METH_VARARGS,
+     "Return CPU statistics"},
 #if defined(__FreeBSD__) || defined(__NetBSD__)
     {"net_connections", psutil_net_connections, METH_VARARGS,
      "Return system-wide open connections."},
