@@ -247,13 +247,15 @@ psutil_linux_sysinfo(PyObject *self, PyObject *args) {
         return PyErr_SetFromErrno(PyExc_OSError);
     // note: boot time might also be determined from here
     return Py_BuildValue(
-        "(KKKKKK)",
-        (unsigned long long)info.totalram  * info.mem_unit,   // total
-        (unsigned long long)info.freeram   * info.mem_unit,   // free
-        (unsigned long long)info.bufferram * info.mem_unit,   // buffer
-        (unsigned long long)info.sharedram * info.mem_unit,   // shared
-        (unsigned long long)info.totalswap * info.mem_unit,   // swap tot
-        (unsigned long long)info.freeswap  * info.mem_unit);  // swap free
+        "(kkkkkkI)",
+        info.totalram,  // total
+        info.freeram,  // free
+        info.bufferram, // buffer
+        info.sharedram, // shared
+        info.totalswap, // swap tot
+        info.freeswap,  // swap free
+        info.mem_unit  // multiplier
+    );
 }
 
 
@@ -273,7 +275,7 @@ psutil_proc_cpu_affinity_get(PyObject *self, PyObject *args) {
     cpu_set_t *mask = NULL;
     PyObject *py_list = NULL;
 
-    if (!PyArg_ParseTuple(args, "i", &pid))
+    if (!PyArg_ParseTuple(args, "l", &pid))
         return NULL;
     ncpus = NCPUS_START;
     while (1) {
@@ -340,7 +342,7 @@ psutil_proc_cpu_affinity_get(PyObject *self, PyObject *args) {
     PyObject* py_retlist = NULL;
     PyObject *py_cpu_num = NULL;
 
-    if (!PyArg_ParseTuple(args, "i", &pid))
+    if (!PyArg_ParseTuple(args, "l", &pid))
         return NULL;
 	CPU_ZERO(&cpuset);
     if (sched_getaffinity(pid, len, &cpuset) < 0)
@@ -364,7 +366,7 @@ psutil_proc_cpu_affinity_get(PyObject *self, PyObject *args) {
 
 error:
     Py_XDECREF(py_cpu_num);
-    Py_DECREF(py_retlist);
+    Py_XDECREF(py_retlist);
     return NULL;
 }
 #endif
